/**
 * @file dmxprogram.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Represents a DMX program
**/
#include "dmxprogram.h"
#include <stdlib.h>
#include <string.h>






/**
 * dmxprogram_create()
 * Creates a new dmxprogram_t object
 * @return Created object or 0 on allocation error
*/
dmxprogram_t *dmxprogram_create()
{
	dmxprogram_t *prog = malloc(sizeof(dmxprogram_t));
	if(prog == 0) return 0;
	/* set inital values */
	prog->data    = 0;
	prog->pos     = 0;
	prog->steplen = 1;
	prog->active  = 0;
	prog->size    = 0;
	prog->trigger = 1;
	prog->group   = 0;
	prog->fade    = 0;
	strcpy(prog->name, "unnamed");
	return prog;
}



/**
 * dmxprogram_destroy()
 * Cleans up
 * @param prog   Pointer to the dmxprogram
*/
void dmxprogram_destroy(dmxprogram_t *prog)
{
	int n;
	if(prog == 0) return;
	/* cleanup data */
	for(n=0; n<prog->size; n++) 
		dmxcue_destroy(prog->data[n]);
	/* free array */
	free(prog->data);
}



/**
 * dmxprogram_append()
 * Appends a cue to cue list
 * @param cue    Pointer to the cue to append
 * @param prog   Pointer to the dmxprogram
 * @return 1 on success, 0 on failure
*/
int dmxprogram_append(dmxlist_t *cue, dmxprogram_t *prog)
{
	int n;
	if(prog == 0) return 0;
	dmxlist_t **tmp = prog->data;
	/* allocate new memory */
	prog->data = (dmxlist_t**) calloc((prog->size+1), sizeof(dmxprogram_t));
	/* copy data */
	for(n=0; n<prog->size; n++) 
		prog->data[n] = tmp[n];
	/* free tmp */
	free(tmp);
	/* append cue */
	prog->data[prog->size] = cue;
	/* increase size */
	prog->size++;	
	return 1;
}



/**
 * dmxprogram_append_new()
 * Appends a empty, new cue to cue list
 * @param prog   Pointer to the dmxprogram
 * @return 1 on success, 0 on failure
*/
int dmxprogram_append_new(dmxprogram_t *prog)
{
	int n;
	if(prog == 0) return 0;
	dmxlist_t **tmp = prog->data;
	dmxlist_t *cue = dmxcue_init();
	/* allocate new memory */
	prog->data = (dmxlist_t**) calloc((prog->size+1), sizeof(dmxprogram_t));
	/* copy data */
	for(n=0; n<prog->size; n++) 
		prog->data[n] = tmp[n];
	/* free tmp */
	free(tmp);
	/* append cue */
	prog->data[prog->size] = cue;
	/* increase size */
	prog->size++;	
	return 1;
}



/**
 * dmxprogram_get_cue()
 * Get a cue by index
 * @param i      Cue index
 * @param prog   Pointer to the dmxprogram
 * @return dmxlist_t* on success, 0 on failure
*/
dmxlist_t *dmxprogram_get_cue(int i, dmxprogram_t *prog)
{
	if(prog == 0) return 0;
	if( (i<0) || (i>=prog->size) ) return 0;
	return prog->data[i];
}



/**
 * dmxprogram_get_current_cue()
 * get current cue
 * @param prog   Pointer to the dmxprogram
 * @return dmxlist_t* on success, 0 on failure
*/
dmxlist_t *dmxprogram_get_current_cue(dmxprogram_t *prog)
{
	if(prog == 0) return 0;
	return dmxprogram_get_cue(prog->pos, prog);
}



/**
 * dmxprogram_step_up()
 * @param prog   Pointer to the dmxprogram
 * go one step up
*/
void dmxprogram_step_up(dmxprogram_t *prog)
{
	if(prog == 0) return;
	prog->pos = (prog->pos + 1) % prog->size;
}



/**
 * dmxprogram_clear()
 * Clears a program
 * @param prog   Pointer to the dmxprogram
*/
void dmxprogram_clear(dmxprogram_t *prog)
{
	int n;
	if(prog == 0) return;
	/* clear all cue data */
	if(prog->data != 0) 
	{
		for(n=0; n<(prog->size); n++)
		{
			dmxcue_clear(prog->data[n]);
		}
		free(prog->data);
	}
	/* reset data info */
	prog->data    = 0;
	prog->pos     = 0;
	prog->steplen = 1;
	prog->active  = 0;
	prog->size    = 0;
	prog->trigger = 1;
	prog->group   = 0;
	prog->fade    = 100;
	strcpy(prog->name, "unnamed");
}



/**
 * Saves program into a file
 * @param fp    File pointer to store data in
 * @param p     Program pointer
 * @return Number of bytes written
**/
long dmxprogram_save(FILE *fp, dmxprogram_t *p)
{
	int n;
	long len;
	/* write main struct */
	len = fwrite(p, sizeof(dmxprogram_t), 1, fp);
	/* write cue data */
	for(n=0; n<(p->size); n++)
		len += dmxcue_save(fp, p->data[n]);
	return len;
}



/**
 * Reads program from a file
 * @param fp   File pointer to read data from
 * @param p    Program pointer
 * @return Number of bytes read
**/
long dmxprogram_load(FILE *fp, dmxprogram_t *p)
{
	int n;
	long len;
	/* clear program */
	dmxprogram_clear(p);
	/* read main scruct */
	len = fread(p, sizeof(dmxprogram_t), 1, fp);
	if(len <=0) return 0;
	/* allocate memory */
	p->data = (dmxlist_t**) calloc(p->size, sizeof(dmxprogram_t));
	/* Load cues */
	for(n=0; n<(p->size); n++)
	{
		p->data[n] = dmxcue_init();
		len += dmxcue_load(fp, p->data[n]);
	}
	return len;
}



/**
 * dmxprogram_speed_set()
 * Sets the program's speed
 * @param prog     Pointer to the dmxprogram
 * @param steplen  Step speed to set program to
*/
void dmxprogram_speed_set(dmxprogram_t *prog, long int steplen)
{
	if(prog == 0) return;
	/* Check & Set speed */
	if(steplen > DMXPROGRAM_STEPLEN_MAX)
		steplen = DMXPROGRAM_STEPLEN_MAX;
	if(steplen < DMXPROGRAM_STEPLEN_MIN)
		steplen = DMXPROGRAM_STEPLEN_MIN;
	prog->steplen = steplen;
}



/**
 * dmxprogram_speed_decrease()
 * Decreases speed
 * @param prog     Pointer to the dmxprogram
*/
void dmxprogram_speed_decrease(dmxprogram_t *prog)
{
	if(prog == 0) return;
	/* Check & Set speed */
	if(prog->steplen > 0)
	{
		dmxprogram_speed_set( prog, (prog->steplen + 1) );
	}
	else
	{
		if(prog->steplen == -2)
		{
			prog->steplen = 1;
		}
		else
		{
			dmxprogram_speed_set( prog, (prog->steplen / 2) );
		}
	}
}



/**
 * dmxprogram_speed_increase()
 * Increases speed
 * @param prog     Pointer to the dmxprogram
*/
void dmxprogram_speed_increase(dmxprogram_t *prog)
{
	if(prog == 0) return;
	/* Check & Set speed */
	if(prog->steplen > 0)
	{
		if(prog->steplen == 1)
		{
			prog->steplen = -2;
		}
		else
		{
			dmxprogram_speed_set( prog, (prog->steplen - 1) );
		}
	}
	else
	{
		dmxprogram_speed_set( prog, (prog->steplen * 2) );
	}
}



/**
 * dmxprogram_set_name()
 * Increases speed
 * @param prog     Pointer to the dmxprogram
 * @param name     Pointer to the new name, is copied
*/
void dmxprogram_set_name(dmxprogram_t *prog, const char *name)
{
	int n;
	if(prog == 0) return;
	/* check line end */
	for(n=0; n<DMXPROGRAM_NAME_LEN; n++)
	{
		if( (name[n] == '\r') || (name[n] == '\n') || (name[n] == '\0') )
		{
			prog->name[n] = '\0';
			break;
		}
		else
		{
			prog->name[n] = name[n];
		}
	}
	/* ensure string end */
	prog->name[DMXPROGRAM_NAME_LEN-1] = '\0';
}
